BANNED_GROUPS = [
    "evo",
    "3lton",
    "4yeo",
    "ade",
    "afg",
    "anihls",
    "animerg",
    "aniurl",
    "aroma",
    "axxo",
    "brrip",
    "chd",
    "cm8",
    "crewsade",
    "d3g",
    "ddr",
    "dnl",
    "deadfish",
    "dnl",
    "elite",
    "esc",
    "fangding0",
    "fgt",
    "frds",
    "fum",
    "haiku",
    "hd2dvd",
    "hds",
    "hdtime",
    "hi10",
    "ion10",
    "iplanet",
    "jive",
    "kingdom",
    "leffe",
    "ligas",
    "load",
    "megusta",
    "mkvcage",
    "mhd",
    "msd",
    "nhanc3",
    "nhd",
    "noivtc",
    "nsd",
    "oj",
    "ozlem",
    "pirates",
    "prodji",
    "rapidcows",
    "rarbg",
    "retropeeps",
    "rdn",
    "resurrection",
    "rmteam",
    "santi",
    "sicfoi",
    "spasm",
    "spdvd",
    "stuttershit",
    "tbs",
    "telly",
    "tm",
    "upinsmoke",
    "uranime",
    "waf",
    "xred",
    "xs",
    "yify",
    "yts",
    "zeus",
    "zkbl",
    "zmn",
    "zmnt",
]
